package com.ruoyi.bst.attachClassify.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.github.pagehelper.PageHelper;
import com.ruoyi.bst.attachClassify.domain.AttachClassify;
import com.ruoyi.bst.attachClassify.domain.AttachClassifyQuery;
import com.ruoyi.bst.attachClassify.domain.AttachClassifyVO;
import com.ruoyi.bst.attachClassify.domain.vo.AttachClassifyNameVO;
import com.ruoyi.bst.attachClassify.mapper.AttachClassifyMapper;
import com.ruoyi.bst.attachClassify.service.AttachClassifyService;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;

/**
 * 资源分类Service业务层处理
 *
 * @author ruoyi
 * @date 2025-02-12
 */
@Service
public class AttachClassifyServiceImpl implements AttachClassifyService
{
    @Autowired
    private AttachClassifyMapper attachClassifyMapper;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询资源分类
     *
     * @param id 资源分类主键
     * @return 资源分类
     */
    @Override
    public AttachClassifyVO selectAttachClassifyById(Long id)
    {
        return attachClassifyMapper.selectAttachClassifyById(id);
    }

    /**
     * 查询资源分类列表
     *
     * @param attachClassify 资源分类
     * @return 资源分类
     */
    @Override
    public List<AttachClassifyVO> selectAttachClassifyList(AttachClassifyQuery attachClassify)
    {
        return attachClassifyMapper.selectAttachClassifyList(attachClassify);
    }

    /**
     * 新增资源分类
     *
     * @param attachClassify 资源分类
     * @return 结果
     */
    @Override
    public int insertAttachClassify(AttachClassify attachClassify)
    {
        // 查询父级分类信息
        AttachClassifyVO parentClassify = attachClassifyMapper.selectAttachClassifyById(attachClassify.getParentId());
        // 如果父节点不为空，设置祖级列表
        if (parentClassify != null) {
            attachClassify.setAncestors(String.valueOf(attachClassify.getParentId()));
        } else {
            attachClassify.setAncestors("0");
        }
        
        attachClassify.setCreateTime(DateUtils.getNowDate());
        
        return transactionTemplate.execute(status -> {
            int insert = attachClassifyMapper.insertAttachClassify(attachClassify);
            if (insert > 0) {
                clearCache();
            }
            return insert;
        });
    }

    /**
     * 修改资源分类
     *
     * @param attachClassify 资源分类
     * @return 结果
     */
    @Override
    public int updateAttachClassify(AttachClassify attachClassify)
    {
        // 查询新的父级分类
        AttachClassifyVO newParentClassify = attachClassifyMapper.selectAttachClassifyById(attachClassify.getParentId());
        // 查询旧的分类信息
        AttachClassifyVO oldClassify = attachClassifyMapper.selectAttachClassifyById(attachClassify.getId());
        
        return transactionTemplate.execute(status -> {
            if (newParentClassify != null && oldClassify != null) {
                // 构建新的祖级列表
                String newAncestors = newParentClassify.getAncestors() + "," + newParentClassify.getId();
                String oldAncestors = oldClassify.getAncestors();
                attachClassify.setAncestors(newAncestors);
                
                // 更新子节点的祖级列表
                updateChildrenAncestors(attachClassify.getId(), newAncestors, oldAncestors);
            }
            
            int update = attachClassifyMapper.updateAttachClassify(attachClassify);
            if (update > 0) {
                clearCache();
            }
            return update;
        });
    }

    /**
     * 更新子节点的祖级列表
     *
     * @param classifyId 分类ID
     * @param newAncestors 新的祖级列表
     * @param oldAncestors 旧的祖级列表
     */
    private void updateChildrenAncestors(Long classifyId, String newAncestors, String oldAncestors)
    {
        List<AttachClassifyVO> children = attachClassifyMapper.selectChildrenById(classifyId);
        for (AttachClassifyVO child : children){
            child.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
        }
        if (!children.isEmpty()){
            attachClassifyMapper.updateChildrenAncestors(children);
        }
    }

    /**
     * 清除缓存
     */
    private void clearCache() {
        redisCache.deleteObject(CacheConstants.ATTACH_CLASSIFY_NAME_LIST);
    }

    /**
     * 批量删除资源分类
     *
     * @param ids 需要删除的资源分类主键
     * @return 结果
     */
    @Override
    public int logicDel(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return 0;
        }

        // 判断分类下是否有子分类
        int childrenCount = this.selectCountByParentIds(ids);
        ServiceUtil.assertion(childrenCount > 0, "ID为" + ids + "的分类下有子分类，不能删除");

        // 不允许删除根分类
        ServiceUtil.assertion(ids.contains(1L), "不允许删除根目录");

        Integer result = transactionTemplate.execute(status -> {
            int delete = attachClassifyMapper.logicDel(ids);
            if (delete > 0) {
                clearCache();
            }
            return delete;
        });

        return result == null ? 0 : result;
    }

    private int selectCountByParentIds(List<Long> parentIds) {   
        if (CollectionUtils.isEmptyElement(parentIds)) {
            return 0;
        }

        AttachClassifyQuery query = new AttachClassifyQuery();
        query.setParentIds(parentIds);
        return this.selectCount(query);
    }

    private int selectCount(AttachClassifyQuery query) {
        return attachClassifyMapper.selectCount(query);
    }

    @Override
    public List<AttachClassifyNameVO> selectAllNameList(){
        List<AttachClassifyNameVO> list = redisCache.getCacheList(CacheConstants.ATTACH_CLASSIFY_NAME_LIST);
        if(CollectionUtils.isEmptyElement(list)){
            PageHelper.orderBy("bac.sort asc");
            list = attachClassifyMapper.selectNameList(new AttachClassifyQuery());
            if (CollectionUtils.isNotEmptyElement(list)) {
                redisCache.setCacheList(CacheConstants.ATTACH_CLASSIFY_NAME_LIST, list);
            }
        }
        return list;
    }
}
