package com.ruoyi.bst.project.service.impl;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.bst.project.domain.Project;
import com.ruoyi.bst.project.domain.ProjectQuery;
import com.ruoyi.bst.project.domain.ProjectVO;
import com.ruoyi.bst.project.domain.dto.ProjectMaintenanceDTO;
import com.ruoyi.bst.project.domain.dto.ProjectStartDTO;
import com.ruoyi.bst.project.domain.enums.ProjectStatus;
import com.ruoyi.bst.project.domain.vo.ProjectNameVO;
import com.ruoyi.bst.project.mapper.ProjectMapper;
import com.ruoyi.bst.project.service.ProjectService;
import com.ruoyi.bst.project.service.ProjectValidator;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.SnowFlakeUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.vo.IntegerIntegerVO;
import com.ruoyi.common.vo.StringIntegerVO;

/**
 * 项目Service业务层处理
 *
 * @author ruoyi
 * @date 2025-01-22
 */
@Service
public class ProjectServiceImpl implements ProjectService
{
    @Autowired
    private ProjectMapper projectMapper;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private ProjectValidator projectValidator;

    /**
     * 查询项目
     *
     * @param id 项目主键
     * @return 项目
     */
    @Override
    public ProjectVO selectProjectById(Long id)
    {
        return projectMapper.selectProjectById(id);
    }

    /**
     * 查询项目列表
     *
     * @param project 项目
     * @return 项目
     */
    @Override
    public List<ProjectVO> selectProjectList(ProjectQuery project)
    {
        return projectMapper.selectProjectList(project);
    }

    /**
     * 新增项目
     *
     * @param project 项目
     * @return 结果
     */
    @Override
    public int insertProject(Project project)
    {
        project.setCreateTime(DateUtils.getNowDate());
        project.setNo(String.valueOf(SnowFlakeUtil.newId()));

        Integer result = transactionTemplate.execute(status -> {
            int insert = projectMapper.insertProject(project);
            if (insert > 0) {
                // 后校验
                ProjectVO vo = this.selectProjectById(project.getId());
                projectValidator.validate(vo);

                // 清除缓存
                clearCache();
            }
            return insert;
        });

        return result == null ? 0 : result;
    }

    /**
     * 修改项目
     *
     * @param project 项目
     * @return 结果
     */
    @Override
    public int updateProject(Project project)
    {
        Integer result = transactionTemplate.execute(status -> {
            int update = projectMapper.updateProject(project);
            if (update > 0) {
                // 后校验
                ProjectVO vo = this.selectProjectById(project.getId());
                projectValidator.validate(vo);

                clearCache();
            }
            return update;
        });

        return result == null ? 0 : result;
    }

    /**
     * 批量删除项目
     *
     * @param ids 需要删除的项目主键
     * @return 结果
     */
    @Override
    public int logicDel(List<Long> ids)
    {
        int result = projectMapper.logicDel(ids);
        if (result > 0) {
            clearCache();
        }
        return result;
    }

    @Override
    public List<ProjectNameVO> selectAllNameList() {
        List<ProjectNameVO> list = redisCache.getCacheList(CacheConstants.PROJECT_NAME_LIST);
        if (CollectionUtils.isEmptyElement(list)) {
            list = projectMapper.selectNameList(new ProjectQuery());
            if (CollectionUtils.isNotEmptyElement(list)) {
                redisCache.setCacheList(CacheConstants.PROJECT_NAME_LIST, list);
            }
        }
        return list;
    }
    
    @Override
    public int selectCount(ProjectQuery query) {
        return projectMapper.selectCount(query);
    }

    /**
     * 清除缓存
     */
    private void clearCache() {
        redisCache.deleteObject(CacheConstants.PROJECT_NAME_LIST);
    }

    @Override
    public List<StringIntegerVO> selectCountGroupByStatus(ProjectQuery query) {
        return projectMapper.selectCountGroupByStatus(query);
    }

    @Override
    public int complete(Long id) {
        // 查询旧数据
        ProjectVO old = this.selectProjectById(id);
        ServiceUtil.assertion(old == null, "ID为%s的项目不存在", id);
        ServiceUtil.assertion(!ProjectStatus.canComplete().contains(old.getStatus()), "ID为%s的项目当前状态不能完成", id);

        // 更新数据
        Project data = new Project();
        data.setStatus(ProjectStatus.COMPLETED.getStatus());
        data.setCompleteTime(LocalDateTime.now());
        ProjectQuery query = new ProjectQuery();
        query.setId(id);
        query.setStatusList(ProjectStatus.canComplete());
        return this.updateByQuery(data, query);
    }

    private int updateByQuery(Project data, ProjectQuery query) {
        return projectMapper.updateByQuery(data, query);
    }

    @Override
    public int start(ProjectStartDTO dto) {
        ProjectVO old = this.selectProjectById(dto.getId());
        ServiceUtil.assertion(old == null, "ID为%s的项目不存在", dto.getId());
        ServiceUtil.assertion(!ProjectStatus.canStart().contains(old.getStatus()), "ID为%s的项目当前状态不能开始", dto.getId());

        Project data = new Project();
        data.setStatus(ProjectStatus.IN_PROGRESS.getStatus());
        data.setExpectedCompleteDate(dto.getExpectedCompleteDate());
        data.setStartTime(LocalDateTime.now());
        ProjectQuery query = new ProjectQuery();
        query.setId(dto.getId());
        query.setStatusList(ProjectStatus.canStart());
        return this.updateByQuery(data, query);
    }

    @Override
    public int accept(Long id) {
        ProjectVO old = this.selectProjectById(id);
        ServiceUtil.assertion(old == null, "ID为%s的项目不存在", id);
        ServiceUtil.assertion(!ProjectStatus.canAccept().contains(old.getStatus()), "ID为%s的项目当前状态不能验收", id);

        Project data = new Project();
        data.setStatus(ProjectStatus.ACCEPTED.getStatus());
        data.setAcceptTime(LocalDateTime.now());
        ProjectQuery query = new ProjectQuery();
        query.setId(id);
        query.setStatusList(ProjectStatus.canAccept());
        return this.updateByQuery(data, query);
    }

    @Override
    public int maintenance(ProjectMaintenanceDTO dto) {
        ProjectVO old = this.selectProjectById(dto.getId());
        ServiceUtil.assertion(old == null, "ID为%s的项目不存在", dto.getId());
        ServiceUtil.assertion(!ProjectStatus.canMaintenance().contains(old.getStatus()), "ID为%s的项目当前状态不能维护", dto.getId());

        Project data = new Project();
        data.setStatus(ProjectStatus.MAINTENANCE.getStatus());
        data.setMaintenanceEndDate(dto.getMaintenanceEndDate());
        data.setOperationAmount(dto.getOperationAmount());
        ProjectQuery query = new ProjectQuery();
        query.setId(dto.getId());
        query.setStatusList(ProjectStatus.canMaintenance());
        return this.updateByQuery(data, query);
    }

    @Override
    public List<IntegerIntegerVO> selectCountGroupByCreateMonth(ProjectQuery query) {
        return projectMapper.selectCountGroupByCreateMonth(query);
    }

    @Override
    public List<IntegerIntegerVO> selectCountGroupByCompleteMonth(ProjectQuery query) {
        return projectMapper.selectCountGroupByCompleteMonth(query);
    }

    @Override
    public List<IntegerIntegerVO> selectCountGroupByStartMonth(ProjectQuery query) {
        return projectMapper.selectCountGroupByStartMonth(query);
    }

    @Override
    public int handleDevOverdueProject(ProjectQuery query) {
        // 更新数据
        Project data = new Project();
        data.setDevOverdue(true);
        data.setStatus(ProjectStatus.DEVELOPMENT_OVERDUE.getStatus());

        // 查询所有未完成开发的项目
        query.setStatusList(ProjectStatus.devNotFinish());
        query.setExpectedCompleteDateEnd(LocalDate.now());
        query.setDevOverdue(false);
        return this.updateByQuery(data, query);
    }

    @Override
    public int handleMaintenanceOverdueProject(ProjectQuery query) {
        // 更新数据
        Project data = new Project();
        data.setStatus(ProjectStatus.MAINTENANCE_OVERDUE.getStatus());

        // 查询所有未完成维护的项目
        query.setStatusList(ProjectStatus.canMaintenanceOverdue());
        query.setMaintenanceEndDateEnd(LocalDate.now());
        return this.updateByQuery(data, query);
    }
}
