package com.ruoyi.bst.task.service.impl;

import java.time.LocalDateTime;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.bst.message.domain.Message;
import com.ruoyi.bst.message.service.MessageConverter;
import com.ruoyi.bst.message.service.MessageService;
import com.ruoyi.bst.task.domain.Task;
import com.ruoyi.bst.task.domain.TaskQuery;
import com.ruoyi.bst.task.domain.TaskVO;
import com.ruoyi.bst.task.domain.dto.TaskCancelDTO;
import com.ruoyi.bst.task.domain.dto.TaskSubmitDTO;
import com.ruoyi.bst.task.domain.enums.TaskStatus;
import com.ruoyi.bst.task.mapper.TaskMapper;
import com.ruoyi.bst.task.service.TaskService;
import com.ruoyi.bst.task.service.TaskValidator;
import com.ruoyi.bst.taskSubmit.domain.TaskSubmit;
import com.ruoyi.bst.taskSubmit.domain.TaskSubmitQuery;
import com.ruoyi.bst.taskSubmit.domain.TaskSubmitVO;
import com.ruoyi.bst.taskSubmit.domain.enums.TaskSubmitStatus;
import com.ruoyi.bst.taskSubmit.service.TaskSubmitConverter;
import com.ruoyi.bst.taskSubmit.service.TaskSubmitService;
import com.ruoyi.bst.verify.service.VerifyConverter;
import com.ruoyi.bst.verify.service.VerifyService;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.vo.LongIntegerVO;
import com.ruoyi.common.vo.StringIntegerVO;

import lombok.extern.slf4j.Slf4j;

/**
 * 任务Service业务层处理
 *
 * @author ruoyi
 * @date 2025-01-22
 */
@Service
@Slf4j
public class TaskServiceImpl implements TaskService
{
    @Autowired
    private TaskMapper taskMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private VerifyService verifyService;

    @Autowired
    private VerifyConverter verifyConverter;

    @Autowired
    private TaskSubmitConverter taskSubmitConverter;

    @Autowired
    private TaskSubmitService taskSubmitService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private TaskValidator taskValidator;

    @Autowired
    private MessageService MessageService;

    @Autowired
    private MessageConverter messageConverter;

    /**
     * 查询任务
     *
     * @param id 任务主键
     * @return 任务
     */
    @Override
    public TaskVO selectTaskById(Long id)
    {
        if (id == null) {
            return null;
        }
        return taskMapper.selectTaskById(id);
    }

    /**
     * 查询任务列表
     *
     * @param task 任务
     * @return 任务
     */
    @Override
    public List<TaskVO> selectTaskList(TaskQuery task)
    {
        return taskMapper.selectTaskList(task);
    }

    /**
     * 新增任务
     *
     * @param task 任务
     * @return 结果
     */
    @Override
    public int insertTask(Task task)
    {
        task.setCreateTime(DateUtils.getNowDate());
        Integer result = transactionTemplate.execute(status -> {
            int rows = taskMapper.insertTask(task);
            if (rows > 0) {
                TaskVO vo = this.selectTaskById(task.getId());
                taskValidator.validate(vo);

                List<Message> messages = messageConverter.toPoByTaskCreate(vo);
                int messageRows = MessageService.batchInsert(messages);
                ServiceUtil.assertion(messageRows != messages.size(), "新增消息失败");
            }
            return rows;
        });

        return result == null ? 0 : result;
    }

    /**
     * 修改任务
     *
     * @param task 任务
     * @return 结果
     */
    @Override
    public int updateTask(Task task)
    {
        Integer result = transactionTemplate.execute(status -> {
            int rows = taskMapper.updateTask(task);

            if (rows > 0) {
                TaskVO vo = this.selectTaskById(task.getId());
                taskValidator.validate(vo);

                // 保存并发送消息
                List<Message> messages = messageConverter.toPoByTaskUpdate(vo);
                int messageRows = MessageService.batchInsert(messages);
                ServiceUtil.assertion(messageRows != messages.size(), "保存消息失败");
                
            }

            return rows;
        });

        return result == null ? 0 : result;
    }

    /**
     * 批量删除任务
     *
     * @param ids 需要删除的任务主键
     * @return 结果
     */
    @Override
    public int logicDel(List<Long> ids)
    {
        return taskMapper.logicDel(ids);
    }

    @Override
    public List<StringIntegerVO> selectCountGroupByStatus(TaskQuery query) {
        return taskMapper.selectCountGroupByStatus(query);
    }

    @Override
    public int selectCount(TaskQuery query) {
        return taskMapper.selectCount(query);
    }

    @Override
    public List<LongIntegerVO> selectCountGroupByProject(TaskQuery query) {
        return taskMapper.selectCountGroupByProject(query);
    }

    @Override
    public int submitTask(TaskSubmitDTO dto) {
        // 校验
        ServiceUtil.assertion(dto == null, "参数不允许为空");
        ServiceUtil.assertion(dto.getId() == null, "任务ID不允许为空");
        TaskVO old = this.selectTaskById(dto.getId());
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", dto.getId());
        ServiceUtil.assertion(!TaskStatus.canSubmit().contains(old.getStatus()), "ID为%s的任务当前状态不允许提交", dto.getId());

        Integer result = transactionTemplate.execute(status -> {
            // 新增提交记录
            TaskSubmit submit = taskSubmitConverter.toPo(dto);
            int insertSubmit = taskSubmitService.insertTaskSubmit(submit);
            ServiceUtil.assertion(insertSubmit != 1, "新增提交记录失败");

            // 保存并发送消息
            TaskSubmitVO submitVo = taskSubmitService.selectTaskSubmitById(submit.getId());
            List<Message> messages = messageConverter.toPoByTaskSubmit(submitVo);
            int messageRows = MessageService.batchInsert(messages);
            ServiceUtil.assertion(messageRows != messages.size(), "保存消息失败");

            // 尝试更新状态为结束
            try {
                this.passTask(dto.getId());
            } catch (Exception e) {
                log.error("更新ID为{}的任务状态为结束失败:{}", dto.getId(), e.getMessage());
            }

            return insertSubmit;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int updateByQuery(Task data, TaskQuery query) {
        return taskMapper.updateByQuery(data, query);
    }


    @Override
    public int passTask(Long id) {
        // 校验
        ServiceUtil.assertion(id == null, "任务ID不允许为空");

        TaskVO old = this.selectTaskById(id);
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", id);
        ServiceUtil.assertion(!TaskStatus.canPass().contains(old.getStatus()), "ID为%s的任务当前状态不允许完成", id);


        // 查询提交记录，判断是否所有负责人都已经通过了
        if (old.getOwnerIds().size() > 0) {
            TaskSubmitQuery submitQuery = new TaskSubmitQuery();
            submitQuery.setTaskId(id);
            submitQuery.setStatus(TaskSubmitStatus.PASS.getCode());
            List<TaskSubmitVO> submitList = taskSubmitService.selectTaskSubmitList(submitQuery);
            List<Long> passedOwnerIds = CollectionUtils.map(submitList, TaskSubmitVO::getUserId);
            ServiceUtil.assertion(!CollectionUtils.equalsIgnoreOrder(old.getOwnerIds(), passedOwnerIds),
            "ID为%s的任务当前状态不允许完成,还有%s个负责人未提交", id, old.getOwnerIds().size() - passedOwnerIds.size());
        }

        // 更新任务状态
        Integer result = transactionTemplate.execute(status -> {
            Task data = new Task();
            data.setStatus(TaskStatus.PASS.getStatus());
            data.setPassTime(LocalDateTime.now());
            TaskQuery query = new TaskQuery();
            query.setId(id);
            query.setStatusList(TaskStatus.canPass());
            int rows =this.updateByQuery(data, query);

            if (rows > 0) {
                // 保存并发送消息
                TaskVO vo = this.selectTaskById(id);
                List<Message> messages = messageConverter.toPoByTaskPass(vo);
                int messageRows = MessageService.batchInsert(messages);
                ServiceUtil.assertion(messageRows != messages.size(), "保存消息失败");
            }

            return rows;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int cancelTask(TaskCancelDTO dto) {

        // 校验
        ServiceUtil.assertion(dto == null, "参数不允许为空");
        ServiceUtil.assertion(dto.getId() == null, "任务ID不允许为空");

        TaskVO old = this.selectTaskById(dto.getId());
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", dto.getId());
        ServiceUtil.assertion(!TaskStatus.canCancel().contains(old.getStatus()), "ID为%s的任务当前状态不允许取消", dto.getId());

        Integer result = transactionTemplate.execute(status -> {
            // 更新任务状态
            Task data = new Task();
            data.setStatus(TaskStatus.CANCEL.getStatus());
            data.setCancelTime(LocalDateTime.now());
            data.setCancelRemark(dto.getCancelRemark());
            data.setCancelUserId(SecurityUtils.getUserId());
            TaskQuery query = new TaskQuery();
            query.setId(dto.getId());
            query.setStatusList(TaskStatus.canCancel());
            int rows = this.updateByQuery(data, query);

            if (rows > 0) {
                // 保存并发送消息
                TaskVO vo = this.selectTaskById(dto.getId());
                List<Message> messages = messageConverter.toPoByTaskCancel(vo);
                int messageRows = MessageService.batchInsert(messages);
                ServiceUtil.assertion(messageRows != messages.size(), "保存消息失败");
            }

            return rows;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int startTask(Long taskId) {
        // 校验
        ServiceUtil.assertion(taskId == null, "任务ID不允许为空");
        TaskVO old = this.selectTaskById(taskId);
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", taskId);
        ServiceUtil.assertion(!TaskStatus.canStart().contains(old.getStatus()), "ID为%s的任务当前状态不允许开始", taskId);

        // 更新任务状态
        Task data = new Task();
        data.setStatus(TaskStatus.PROCESSING.getStatus());
        TaskQuery query = new TaskQuery();
        query.setId(taskId);
        query.setStatusList(TaskStatus.canStart());
        return this.updateByQuery(data, query);
    }

    @Override
    public List<StringIntegerVO> selectCountGroupByType(TaskQuery query) {
        return taskMapper.selectCountGroupByType(query);
    }
}
